
# q-expansion of newform 2268.2.j.n, downloaded from the LMFDB on 26 April 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 2268
weight = 2
poly_data = [1, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[1135, [1, 0]], [1541, [0, -1]], [325, [1, 0]]]
aps_data = [[0, 0], [0, 0], [0, 4], [1, -1], [2, -2], [0, 6], [4, 0], [-4, 0], [0, 2], [-2, 2], [0, 0], [2, 0], [0, 0], [4, -4], [12, -12], [6, 0], [0, -8], [-6, 6], [0, 8], [-14, 0], [-2, 0], [-12, 12], [-4, 4], [0, 0], [2, -2], [16, -16], [0, 16], [-18, 0], [-2, 0], [0, 10], [12, 0], [0, 4], [-2, 2], [0, -4], [0, -6], [8, -8], [0, -14], [-16, 0], [0, 4], [16, -16], [18, 0], [-6, 0], [-18, 18], [0, 10], [22, 0], [-16, 0], [0, 4], [-8, 8], [0, 0], [0, -10], [-14, 0], [0, -6], [-14, 14], [24, 0], [0, -24], [18, -18], [20, 0], [24, 0], [22, -22], [-10, 10], [0, -4], [0, 0], [4, 0], [0, 4], [-10, 10], [18, -18], [28, -28], [0, 6], [0, 6], [-22, 22], [12, -12], [2, 0], [16, -16], [0, -22], [4, 0], [0, 8], [22, -22], [-18, 0], [0, -18], [0, -38], [0, 0], [6, -6], [-30, 0], [26, 0], [24, -24], [38, -38], [14, 0], [-10, 10], [40, -40], [0, -4], [8, 0], [-28, 28], [-16, 0], [0, -42], [0, 20], [24, 0], [0, -36], [-12, 0], [-28, 0], [-30, 0], [-24, 24], [38, 0], [0, 8], [30, -30], [0, 24], [-38, 0], [32, -32], [-12, 0], [0, 6], [46, -46], [0, 0], [6, 0], [0, 38], [20, -20], [20, 0], [-10, 10], [0, -44], [-12, 0], [0, 6], [2, -2], [0, -22], [34, -34], [-24, 24], [-30, 0], [28, -28], [50, 0], [-38, 38], [0, 0], [-16, 16], [0, 6], [0, 0], [0, 30], [0, 0], [-42, 0], [0, 32], [0, 38], [8, 0], [0, 12], [0, 0], [-10, 0], [28, 0], [2, -2], [0, 20], [-18, 0], [-14, 0], [44, -44], [-14, 14], [-48, 48], [0, -44], [14, 0], [0, -46], [-36, 0], [44, 0], [0, -12], [-12, 12], [-18, 18], [16, 0], [48, -48], [2, 0], [0, 28], [10, -10], [22, 0], [0, -20], [-36, 0], [0, 30], [-48, 48], [56, 0], [-38, 38], [14, 0], [0, 0], [-38, 38], [0, 50], [0, -42], [-6, 6], [0, -24], [0, 4], [-48, 48], [-38, 0], [-40, 0], [34, -34], [-36, 36], [20, -20], [0, 38], [20, 0], [0, -62], [24, -24], [6, 0], [-44, 44], [0, 42], [24, 0], [46, 0], [18, -18], [52, 0], [8, -8], [30, 0], [0, -28], [0, 34], [62, -62], [0, 0], [36, 0], [0, -30], [24, -24], [0, -2], [18, -18], [-60, 0], [0, 0], [8, 0], [0, 42], [-6, 6], [0, 52], [2, 0], [0, -20], [48, -48], [4, -4], [0, 36], [-50, 50], [0, 8], [20, -20], [-2, 0], [0, 14], [0, -22], [0, -16], [0, 42], [16, 0], [0, 36], [-50, 50], [32, -32], [-58, 0], [-48, 48], [-66, 66], [0, -30], [-44, 0], [0, 8], [0, -6], [4, -4], [0, 0], [0, 2], [-66, 0], [0, -18], [-40, 0], [-10, 10], [44, 0], [0, 16], [-30, 0], [0, -64], [36, -36], [-48, 0], [0, 32], [0, -8], [-18, 0], [0, -70], [36, 0], [0, -42], [50, -50], [-16, 16], [-50, 0], [26, 0], [36, -36], [-40, 0], [50, 0], [-76, 76], [-74, 74], [0, -10], [22, 0], [0, -60], [44, -44], [-42, 0], [28, -28], [0, -64], [0, 54], [0, 42], [-28, 0], [18, -18], [0, -8], [0, 26], [-48, 0], [0, -22], [-66, 66], [46, 0], [-48, 48], [0, 24], [0, 60], [-48, 48], [10, -10], [0, -36], [-62, 0], [18, 0], [0, 2], [16, -16], [-24, 0], [18, -18], [-84, 0], [0, 54], [0, -32], [34, -34], [0, 0], [-8, 8], [-36, 36], [-24, 0], [36, -36], [0, -10], [18, 0], [-44, 0]]
